==> mongodb-5.0.31.gh.tar.gz: mongo-r5.0.31/src/mongo/db/query/stage_builder.h <==
/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include "mongo/db/exec/working_set.h"
#include "mongo/db/query/query_solution.h"

namespace mongo::stage_builder {
/**
 * The StageBuilder converts a QuerySolution tree to an executable tree of PlanStage(s), with the
 * specific type defined by the 'PlanStageType' parameter.
 */
template <typename PlanStageType>
class StageBuilder {
public:
    StageBuilder(OperationContext* opCtx,
                 const CollectionPtr& collection,
                 const CanonicalQuery& cq,
                 const QuerySolution& solution)
        : _opCtx(opCtx), _collection(collection), _cq(cq), _solution(solution) {}

    virtual ~StageBuilder() = default;

    /**
     * Given a root node of a QuerySolution tree, builds and returns a corresponding executable
     * tree of PlanStages.
     */
    virtual std::unique_ptr<PlanStageType> build(const QuerySolutionNode* root) = 0;

protected:
    OperationContext* _opCtx;
    const CollectionPtr& _collection;
    const CanonicalQuery& _cq;
    const QuerySolution& _solution;
};
}  // namespace mongo::stage_builder

==> mongodb-6.0.27.gh.tar.gz: mongo-r6.0.27/src/mongo/db/query/stage_builder.h <==
/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include "mongo/db/exec/working_set.h"
#include "mongo/db/query/query_solution.h"

namespace mongo::stage_builder {
/**
 * The StageBuilder converts a QuerySolution tree to an executable tree of PlanStage(s), with the
 * specific type defined by the 'PlanStageType' parameter.
 */
template <typename PlanStageType>
class StageBuilder {
public:
    StageBuilder(OperationContext* opCtx, const CanonicalQuery& cq, const QuerySolution& solution)
        : _opCtx(opCtx), _cq(cq), _solution(solution) {}

    virtual ~StageBuilder() = default;

    /**
     * Given a root node of a QuerySolution tree, builds and returns a corresponding executable
     * tree of PlanStages.
     */
    virtual std::unique_ptr<PlanStageType> build(const QuerySolutionNode* root) = 0;

protected:
    OperationContext* _opCtx;
    const CanonicalQuery& _cq;
    const QuerySolution& _solution;
};
}  // namespace mongo::stage_builder
